"use strict";

exports.__esModule = true;
exports.useButtonProps = useButtonProps;
exports.default = void 0;

var React = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _jsxRuntime = require("react/jsx-runtime");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function isTrivialHref(href) {
  return !href || href.trim() === '#';
}

function useButtonProps({
  tagName = 'button',
  disabled,
  href,
  target,
  rel,
  onClick,
  tabIndex = 0
}) {
  if (tagName === 'button') return {
    disabled
  };

  const handleClick = event => {
    if (disabled || isTrivialHref(href)) {
      event.preventDefault();
    }

    if (disabled) {
      event.stopPropagation();
      return;
    }

    onClick == null ? void 0 : onClick(event);
  };

  const handleKeyDown = event => {
    if (event.key === ' ') {
      event.preventDefault();
      handleClick(event);
    }
  };

  return {
    role: 'button',
    tabIndex: disabled ? undefined : tabIndex,
    href: tagName === 'a' && disabled ? undefined : href,
    target: tagName === 'a' ? target : undefined,
    'aria-disabled': !disabled ? undefined : disabled,
    rel: tagName === 'a' ? rel : undefined,
    onClick: handleClick,
    onKeyDown: handleKeyDown
  };
}

const propTypes = {
  /**
   * Disables the Button, preventing mouse events,
   * even if the underlying component is an `<a>` element
   */
  disabled: _propTypes.default.bool,

  /** Providing a `href` will render an `<a>` element, _styled_ as a button. */
  href: _propTypes.default.string,

  /**
   * Defines HTML button type attribute.
   *
   * @default 'button'
   */
  type: _propTypes.default.oneOf(['button', 'reset', 'submit', null]),
  as: _propTypes.default.elementType
};
const Button = /*#__PURE__*/React.forwardRef((_ref, ref) => {
  let {
    type,
    as: Component
  } = _ref,
      props = _objectWithoutPropertiesLoose(_ref, ["type", "as"]);

  const tagName = // eslint-disable-next-line no-nested-ternary
  typeof Component === 'string' ? Component : props.href ? 'a' : 'button';
  const buttonProps = useButtonProps(Object.assign({
    tagName
  }, props));
  Component = Component || tagName;

  if (tagName === 'button' && !type) {
    type = 'button';
  }

  return /*#__PURE__*/(0, _jsxRuntime.jsx)(Component, Object.assign({}, props, buttonProps, {
    ref: ref,
    type: type
  }));
});
Button.displayName = 'Button';
Button.propTypes = propTypes;
var _default = Button;
exports.default = _default;