function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import * as React from 'react';
import PropTypes from 'prop-types';
import { jsx as _jsx } from "react/jsx-runtime";

function isTrivialHref(href) {
  return !href || href.trim() === '#';
}

export function useButtonProps({
  tagName = 'button',
  disabled,
  href,
  target,
  rel,
  onClick,
  tabIndex = 0
}) {
  if (tagName === 'button') return {
    disabled
  };

  const handleClick = event => {
    if (disabled || isTrivialHref(href)) {
      event.preventDefault();
    }

    if (disabled) {
      event.stopPropagation();
      return;
    }

    onClick == null ? void 0 : onClick(event);
  };

  const handleKeyDown = event => {
    if (event.key === ' ') {
      event.preventDefault();
      handleClick(event);
    }
  };

  return {
    role: 'button',
    tabIndex: disabled ? undefined : tabIndex,
    href: tagName === 'a' && disabled ? undefined : href,
    target: tagName === 'a' ? target : undefined,
    'aria-disabled': !disabled ? undefined : disabled,
    rel: tagName === 'a' ? rel : undefined,
    onClick: handleClick,
    onKeyDown: handleKeyDown
  };
}
const propTypes = {
  /**
   * Disables the Button, preventing mouse events,
   * even if the underlying component is an `<a>` element
   */
  disabled: PropTypes.bool,

  /** Providing a `href` will render an `<a>` element, _styled_ as a button. */
  href: PropTypes.string,

  /**
   * Defines HTML button type attribute.
   *
   * @default 'button'
   */
  type: PropTypes.oneOf(['button', 'reset', 'submit', null]),
  as: PropTypes.elementType
};
const Button = /*#__PURE__*/React.forwardRef((_ref, ref) => {
  let {
    type,
    as: Component
  } = _ref,
      props = _objectWithoutPropertiesLoose(_ref, ["type", "as"]);

  const tagName = // eslint-disable-next-line no-nested-ternary
  typeof Component === 'string' ? Component : props.href ? 'a' : 'button';
  const buttonProps = useButtonProps(Object.assign({
    tagName
  }, props));
  Component = Component || tagName;

  if (tagName === 'button' && !type) {
    type = 'button';
  }

  return /*#__PURE__*/_jsx(Component, Object.assign({}, props, buttonProps, {
    ref: ref,
    type: type
  }));
});
Button.displayName = 'Button';
Button.propTypes = propTypes;
export default Button;