import * as React from 'react';
import { EventKey, DynamicRefForwardingComponent, SelectCallback } from './types';
import Button from './Button';
export interface DropdownItemProps extends Omit<React.HTMLAttributes<HTMLElement>, 'onSelect'> {
    as?: React.ElementType;
    active?: boolean;
    disabled?: boolean;
    eventKey?: EventKey;
    href?: string;
    onSelect?: SelectCallback;
}
interface UseDropdownItemOptions {
    key?: EventKey | null;
    active?: boolean;
    disabled?: boolean;
    onClick?: React.MouseEventHandler;
}
/**
 * Create a dropdown item. Returns a set of props for the dropdown item component
 * including an `onClick` handler that prevents selection when the item is disabled
 */
export declare function useDropdownItem({ key, active, disabled, onClick, }: UseDropdownItemOptions): readonly [{
    readonly onClick: (event: any) => void;
    readonly 'aria-disabled': true | undefined;
    readonly 'aria-selected': boolean | undefined;
    readonly "data-rr-ui-dropdown-item": "";
}, {
    readonly active: boolean | undefined;
}];
declare const DropdownItem: DynamicRefForwardingComponent<typeof Button, DropdownItemProps>;
export default DropdownItem;
